@@ -1,7 +1,12 @@
 # -*- coding: utf-8 -*-
 from abc import ABCMeta
-import json
-from flask import Response, render_template
+
+from pathlib import Path
+
+from fastapi import Response
+from fastapi.responses import JSONResponse
+from fastapi.templating import Jinja2Templates
+
 from rdflib import Graph, Namespace, URIRef, BNode, Literal
 from rdflib.namespace import PROF, RDF, RDFS, XSD
 from rdflib.namespace import DCTERMS
@@ -9,6 +14,10 @@ from pyldapi.profile import Profile
 from pyldapi.exceptions import ProfilesMediatypesException
 import re
 import connegp
+from .data import MEDIATYPE_NAMES, RDF_MEDIATYPES
+
+templates_dir = Path(__file__).parent.parent / "pyldapi" / "templates"
+templates = Jinja2Templates(directory=str(templates_dir))
 
 
 class Renderer(object, metaclass=ABCMeta):
@@ -17,31 +26,11 @@ class Renderer(object, metaclass=ABCMeta):
     either registers or objects) and also creates an 'alternates profile' for them, based on all available profiles & mediatypes.
     """
 
-    RDF_MEDIA_TYPES = ['text/turtle', 'application/rdf+xml', 'application/ld+json', 'text/n3', 'application/n-triples']
-    RDF_SERIALIZER_TYPES_MAP = {
-        "text/turtle": "turtle",
-        "text/n3": "n3",
-        "application/n-triples": "nt",
-        "application/ld+json": "json-ld",
-        "application/rdf+xml": "xml",
-        # Some common but incorrect mimetypes
-        "application/rdf": "xml",
-        "application/rdf xml": "xml",
-        "application/json": "json-ld",
-        "application/ld json": "json-ld",
-        "text/ttl": "turtle",
-        "text/ntriples": "nt",
-        "text/n-triples": "nt",
-        "text/plain": "nt",  # text/plain is the old/deprecated mimetype for n-triples
-    }
-
     def __init__(self,
                  request,
                  instance_uri,
                  profiles,
                  default_profile_token,
-                 alternates_template=None,
-                 **kwargs
                  ):
         """
         Constructor
@@ -77,7 +66,7 @@ class Renderer(object, metaclass=ABCMeta):
             'http://www.w3.org/ns/dx/conneg/altr',  # the ConnegP URI for Alt Rep Data Model
             'Alternate Representations',
             'The representation of the resource that lists all other representations (profiles and Media Types)',
-            ['text/html', 'application/json'] + self.RDF_MEDIA_TYPES,
+            ['text/html', 'application/json'] + RDF_MEDIATYPES,
             'text/html',
             languages=['en'],  # default 'en' only for now
         )
@@ -95,9 +84,6 @@ class Renderer(object, metaclass=ABCMeta):
 
         self.default_profile_token = default_profile_token
 
-        # TODO: supply an alternates.html template
-        self.alt_template = alternates_template
-
         # get profile & mediatype for this request, flag any errors but do not except out
         self.profile = self._get_profile()
         self.mediatype = self._get_mediatype()
@@ -132,7 +118,8 @@ class Renderer(object, metaclass=ABCMeta):
         :rtype: list
         """
         # try QSAa and, if we have any, return them only
-        profiles_string = self.request.values.get('_view', self.request.values.get('_profile'))
+        profiles_string = self.request.query_params.get('_view', self.request.query_params.get('_profile'))
+        # profiles_string = None  # TODO: Change request from Flask to FastAPI
         if profiles_string is not None:
             pqsa = connegp.ProfileQsaParser(profiles_string)
             if pqsa.valid:
@@ -214,7 +201,8 @@ class Renderer(object, metaclass=ABCMeta):
         """Returns a list of Media Types from QSA
         :return: list
         """
-        qsa_mediatypes = self.request.values.get('_format', self.request.values.get('_mediatype', None))
+        qsa_mediatypes = self.request.query_params.get('_format', self.request.query_params.get('_mediatype', None))
+        # qsa_mediatypes = None
         if qsa_mediatypes is not None:
             qsa_mediatypes = str(qsa_mediatypes).replace(' ', '+').split(',')
             # if the internal mediatype is requested, return the default
@@ -285,7 +273,8 @@ class Renderer(object, metaclass=ABCMeta):
         """Returns a list of Languages from QSA
         :return: list
         """
-        languages = self.request.values.get('_lang')
+        # languages = self.request.values.get('_lang')
+        languages = None
         if languages is not None:
             languages = str(languages).replace(' ', '_').replace('+', '_').split(',')
             # if the internal mediatype is requested, return the default
@@ -438,20 +427,7 @@ class Renderer(object, metaclass=ABCMeta):
         if headers is None:
             headers = self.headers
 
-        if mimetype is not None:
-            response_mimetype = mimetype
-            serial_mediatype = self.RDF_SERIALIZER_TYPES_MAP.get(mimetype)
-        elif self.mediatype is not None:
-            response_mimetype = self.mediatype
-            if self.mediatype in ['application/rdf+json', 'application/json']:
-                serial_mediatype = 'json-ld'
-            else:
-                serial_mediatype = self.RDF_SERIALIZER_TYPES_MAP.get(self.mediatype)
-        else:
-            serial_mediatype = "turtle"
-            response_mimetype = "text/turtle"
-
-        response_text = graph.serialize(format=serial_mediatype, encoding='utf-8')
+        response_text = graph.serialize(format=mimetype or "text/turtle")
 
         if delete_graph:
             # destroy the triples in the triplestore, then delete the triplestore
@@ -462,11 +438,34 @@ class Renderer(object, metaclass=ABCMeta):
 
         return Response(
             response_text,
-            mimetype=response_mimetype,
+            media_type=mimetype,
             headers=headers
         )
 
-    def _render_alt_profile_html(self, template_context=None):
+    def _render_alt_profile_html(
+            self,
+            alt_template: str = "alt.html",
+            additional_alt_template_context=None,
+            alt_template_context_replace=False
+    ):
+        """Renders the Alternates Profile in HTML
+
+        If an alt_template value (str, file name) is given, a template of that name will be looked for in the app's
+        template directory, else alt.html will be used.
+
+        If a dictionary is given for additional_alt_template_context, this will either replace the standard template
+        context, if alt_template_context_replace is True, or just be added to it, if alt_template_context_replace is
+        False
+
+        :param alt_template: the name of the template to use
+        :type alt_template: str (file name, within the application's templates directory)
+        :param additional_alt_template_context: Additional or complete context for the template
+        :type additional_alt_template_context: dict
+        :param alt_template_context_replace: To replace (True) or add to (False) the standard template context
+        :type alt_template_context_replace: bool
+        :return: a rendered template (HTML)
+        :rtype: TemplateResponse
+        """
         profiles = {}
         for token, profile in self.profiles.items():
             profiles[token] = {
@@ -477,33 +476,36 @@ class Renderer(object, metaclass=ABCMeta):
                 'default_language': str(profile.default_language),
                 'uri': str(profile.uri)
             }
+
         _template_context = {
             'uri': self.instance_uri,
             'default_profile_token': self.default_profile_token,
-            'profiles': profiles
+            'profiles': profiles,
+            'mediatype_names': MEDIATYPE_NAMES,
+            'request': self.request
         }
-        if template_context is not None and isinstance(template_context, dict):
-            _template_context.update(template_context)
-        return Response(
-            render_template(
-                self.alt_template or 'alt.html',
-                **_template_context
-            ),
-            headers=self.headers
-        )
+        if additional_alt_template_context is not None and isinstance(additional_alt_template_context, dict):
+            if alt_template_context_replace:
+                _template_context = additional_alt_template_context
+            else:
+                _template_context.update(additional_alt_template_context)
+
+        return templates.TemplateResponse(alt_template,
+                                          context=_template_context,
+                                          headers=self.headers)
 
     def _render_alt_profile_rdf(self):
         g = self._generate_alt_profiles_rdf()
         return self._make_rdf_response(g)
 
     def _render_alt_profile_json(self):
-        return Response(
-            json.dumps({
+        return JSONResponse(
+            content={
                 'uri': self.instance_uri,
                 'profiles': list(self.profiles.keys()),
                 'default_profile': self.default_profile_token
-            }),
-            mimetype='application/json',
+            },
+            media_type='application/json',
             headers=self.headers
         )
 
@@ -518,7 +520,7 @@ class Renderer(object, metaclass=ABCMeta):
             return self
         if self.mediatype == 'text/html':
             return self._render_alt_profile_html()
-        elif self.mediatype in Renderer.RDF_MEDIA_TYPES:
+        elif self.mediatype in RDF_MEDIATYPES:
             return self._render_alt_profile_rdf()
         else:  # application/json
             return self._render_alt_profile_json()
@@ -538,7 +540,7 @@ class Renderer(object, metaclass=ABCMeta):
 
         # if there's been an error with the request, return that
         if self.vf_error is not None:
-            return Response(self.vf_error, status=400, mimetype='text/plain')
+            return Response(self.vf_error, status=400, media_type='text/plain')
         elif self.profile == 'alt' or self.profile == 'alternates':
             return self._render_alt_profile()
         return None
